<?php
/*
 * This file is part of Phyxo package
 *
 * Copyright(c) Nicolas Roudaire  https://www.phyxo.net/
 * Licensed under the GPL version 2.0 license.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

use App\Repository\CategoryRepository;
use App\Repository\ImageRepository;

// +-----------------------------------------------------------------------+
// | Photo selection                                                       |
// +-----------------------------------------------------------------------+

$upload_max_filesize = min(
    \Phyxo\Functions\Upload::get_ini_size('upload_max_filesize'),
    \Phyxo\Functions\Upload::get_ini_size('post_max_size')
);

if ($upload_max_filesize == \Phyxo\Functions\Upload::get_ini_size('upload_max_filesize')) {
    $upload_max_filesize_shorthand = \Phyxo\Functions\Upload::get_ini_size('upload_max_filesize', false);
} else {
    $upload_max_filesize_shorthand = \Phyxo\Functions\Upload::get_ini_size('post_max_filesize', false);
}

$template->assign(
    [
        'F_ADD_ACTION' => PHOTOS_ADD_BASE_URL,
        'upload_max_filesize' => $upload_max_filesize,
        'upload_max_filesize_shorthand' => $upload_max_filesize_shorthand,
    ]
);

// what is the maximum number of pixels permitted by the memory_limit?
if (\Phyxo\Image\Image::get_library() === 'GD') {
    $fudge_factor = 1.7;
    $available_memory = \Phyxo\Functions\Upload::get_ini_size('memory_limit') - memory_get_usage();
    $max_upload_width = round(sqrt($available_memory / (2 * $fudge_factor)));
    $max_upload_height = round(2 * $max_upload_width / 3);

    // we don't want dimensions like 2995x1992 but 3000x2000
    $max_upload_width = round($max_upload_width / 100) * 100;
    $max_upload_height = round($max_upload_height / 100) * 100;

    $max_upload_resolution = floor($max_upload_width * $max_upload_height / (1000000));

    // no need to display a limitation warning if the limitation is huge like 20MP
    if ($max_upload_resolution < 25) {
        $template->assign(
            [
                'max_upload_width' => $max_upload_width,
                'max_upload_height' => $max_upload_height,
                'max_upload_resolution' => $max_upload_resolution,
            ]
        );
    }
}

//warn the user if the picture will be resized after upload
if ($conf['original_resize']) {
    $template->assign(
        [
            'original_resize_maxwidth' => $conf['original_resize_maxwidth'],
            'original_resize_maxheight' => $conf['original_resize_maxheight'],
        ]
    );
}

$template->assign(
    [
        'form_action' => PHOTOS_ADD_BASE_URL,
        'pwg_token' => \Phyxo\Functions\Utils::get_token(),
    ]
);

$unique_exts = array_unique(
    array_map(
        'strtolower',
        $conf['upload_form_all_types'] ? $conf['file_ext'] : $conf['picture_ext']
    )
);

$template->assign(
    [
        'upload_file_types' => implode(', ', $unique_exts),
        'file_exts' => implode(',', $unique_exts),
    ]
);

// +-----------------------------------------------------------------------+
// | Categories                                                            |
// +-----------------------------------------------------------------------+

// we need to know the category in which the last photo was added
$selected_category = [];

if (isset($_GET['album'])) {
    // set the category from get url or ...
    \Phyxo\Functions\Utils::check_input_parameter('album', $_GET, false, PATTERN_ID);

    // test if album really exists
    $album = (new CategoryRepository($conn))->findById($_GET['album']);
    if (!empty($album)) {
        $selected_category = [$_GET['album']];

        // lets put in the session to persist in case of upload method switch
        $_SESSION['selected_category'] = $selected_category;
    } else {
        \Phyxo\Functions\HTTP::fatal_error('[Hacking attempt] the album id = "' . $_GET['album'] . '" is not valid');
    }
} elseif (isset($_SESSION['selected_category'])) {
    $selected_category = $_SESSION['selected_category'];
} else {
    // we need to know the category in which the last photo was added
    $result = (new ImageRepository($conn))->findCategoryWithLastImageAdded();
    if ($conn->db_num_rows($result) > 0) {
        $row = $conn->db_fetch_assoc($result);
        $selected_category = [$row['category_id']];
    }
}

// existing album
$template->assign('selected_category', $selected_category);


// image level options
$selected_level = isset($_POST['level']) ? $_POST['level'] : 0;
$template->assign(
    [
        'level_options' => \Phyxo\Functions\Utils::get_privacy_level_options(),
        'level_options_selected' => [$selected_level]
    ]
);

// +-----------------------------------------------------------------------+
// | Setup errors/warnings                                                 |
// +-----------------------------------------------------------------------+

// Errors
$setup_errors = [];

$error_message = \Phyxo\Functions\Upload::ready_for_upload_message();
if (!empty($error_message)) {
    $setup_errors[] = $error_message;
}

if (!function_exists('gd_info')) {
    $setup_errors[] = \Phyxo\Functions\Language::l10n('GD library is missing');
}

$template->assign([
    'setup_errors' => $setup_errors,
    'CACHE_KEYS' => \Phyxo\Functions\Utils::get_admin_client_cache_keys(['categories']),
]);

// Warnings
if (isset($_GET['hide_warnings'])) {
    $_SESSION['upload_hide_warnings'] = true;
}

if (!isset($_SESSION['upload_hide_warnings'])) {
    $setup_warnings = [];

    if ($conf['use_exif'] and !function_exists('exif_read_data')) {
        $setup_warnings[] = \Phyxo\Functions\Language::l10n('Exif extension not available, admin should disable exif use');
    }

    if (\Phyxo\Functions\Upload::get_ini_size('upload_max_filesize') > \Phyxo\Functions\Upload::get_ini_size('post_max_size')) {
        $setup_warnings[] = \Phyxo\Functions\Language::l10n(
            'In your php.ini file, the upload_max_filesize (%sB) is bigger than post_max_size (%sB), you should change this setting',
            \Phyxo\Functions\Upload::get_ini_size('upload_max_filesize', false),
            \Phyxo\Functions\Upload::get_ini_size('post_max_size', false)
        );
    }
    $template->assign(
        [
            'setup_warnings' => $setup_warnings,
            'hide_warnings_link' => PHOTOS_ADD_BASE_URL . '&amp;hide_warnings=1'
        ]
    );
}
